@@ -324,24 +324,28 @@ class HgImplementation(M.RepositoryImplementation):
         return sum(1 for _ in self.commits(path=path, rev=rev))
 
     def tarball(self, commit):
-        shortname = self._repo.project.shortname.replace('/', '-')
-        mount_point = self._repo.app.config.options.mount_point
         if not os.path.exists(self._repo.tarball_path):
             os.makedirs(self._repo.tarball_path)
-        archive_name = '%s-%s-%s' % (shortname, mount_point, commit)
-        path = os.path.join(self._repo.tarball_path, archive_name)
-        archive_file = path + '.tar.gz'
+        path = os.path.join(self._repo.tarball_path, commit)
+        archive_name = self._repo.tarball_filename(commit)
+        filename = os.path.join(self._repo.tarball_path, '%s%s' % (archive_name, '.tar.gz'))
+        tmpfilename = os.path.join(self._repo.tarball_path, '%s%s' % (archive_name, '.tmp'))
         if os.path.exists(path):
             rmtree(path)
-        commands.archive(
-            HgUI(),
-            self._hg,
-            path,
-            rev=commit,
-            prefix='')
-        with tarfile.open(archive_file, 'w:gz') as tar:
-            tar.add(path, arcname=archive_name)
-        rmtree(path)
 
+        try:
+            commands.archive(
+                HgUI(),
+                self._hg,
+                path,
+                rev=commit,
+                prefix='')
+            with tarfile.open(tmpfilename, "w:gz") as tar:
+                tar.add(path, arcname=archive_name)
+            os.rename(tmpfilename, filename)
+        finally:
+            rmtree(path)
+            if os.path.exists(tmpfilename):
+                os.remove(tmpfilename)
 
 Mapper.compile_all()
